@@ -23,6 +23,7 @@
 #'   Default: TRUE.
 #' @param cutoff A non-negative number. Segments shorter than this are excluded
 #'   from the output. Default: 0.
+#' @param unit The unit of `cutoff`: either "mb" or "cm".
 #'
 #' @return A matrix (if `sims` is a single `genomeSim` object), or a list of
 #'   matrices.
@@ -32,30 +33,34 @@
 #' @examples
 #' x = nuclearPed(3)
 #' s = ibdsim(x, N = 1, map = uniformMap(M = 1), seed = 1729)
-#' s1 = s[[1]]
-#'
+#' 
 #' # Segments where some allele is shared by 3 and 4, but not 5
 #' pattern = list(carriers = 3:4, noncarriers = 5)
-#' findPattern(s1, pattern)
+#' findPattern(s, pattern)
 #'
 #' # Exclude segments less than 7 cM
-#' findPattern(s1, pattern, cutoff = 7)
+#' findPattern(s, pattern, cutoff = 7)
 #'
 #' # Visual confirmation:
-#' haploDraw(x, s1, margin = c(5,3,3,3))
+#' haploDraw(x, s)
 #'
 #' @export
-findPattern = function(sims, pattern, merge = TRUE, cutoff = 0) {
+findPattern = function(sims, pattern, merge = TRUE, cutoff = 0, unit = "mb") {
   
   if(single <- is.matrix(sims))
     sims = list(sims)
   
-  names(pattern) = match.arg(names(pattern), 
-                             c("autozygous", "heterozygous", "carriers", "noncarriers"), 
-                             several.ok = TRUE)
+  # Allowed names in `pattern` list
+  PNAM = c("autozygous", "heterozygous", "carriers", "noncarriers")
   
-  pattern = lapply(pattern, as.character)
+  # Names occurring in input, expand abbreviations
+  inputPnam = match.arg(names(pattern), PNAM, several.ok = TRUE)
   
+  # Merge if multiple of same type, and make character
+  uniqPnam = unique.default(inputPnam)
+  pattern = sapply(uniqPnam, function(b) pattern[inputPnam == b] |> unlist(use.names = FALSE) |> as.character(), 
+                simplify = FALSE)
+
   aut = pattern$autozygous
   het = pattern$heterozygous
   car = setdiff(pattern$carriers, c(aut, het))
@@ -67,9 +72,11 @@ findPattern = function(sims, pattern, merge = TRUE, cutoff = 0) {
   # All carriers
   allcarr = c(aut, het, car)
   
-  # If no carriers, return. TODO: Why return all of sims???
-  if(!length(allcarr)) 
-    return(sims)
+  # If no carriers, return unbroken chromosomes.
+  if(!length(allcarr)) {
+    res = lapply(sims, function(s) mergeSegments(s)[, 1:5])
+    return(if(single) res[[1]] else res)
+  }
 
   if(length(err <- intersect(non, allcarr)))
     stop2("Individuals indicated as both carrier and noncarrier: ", err)
@@ -162,9 +169,11 @@ findPattern = function(sims, pattern, merge = TRUE, cutoff = 0) {
       s = mergeSegments(s, checkAdjacency = TRUE)
     
     # Apply length cutoff
-    if(cutoff > 0)
-      s = s[s[, 'length'] > cutoff, , drop = FALSE]
-    
+    if(cutoff > 0) {
+      len = switch(unit, mb = s[, 'endMB'] - s[, 'startMB'],
+                   cm = s[, 'endCM'] - s[, 'startCM'])
+      s = s[len >= cutoff, , drop = FALSE]
+    }
     # Result for this sim
     s
   })
